@@ -2,14 +2,15 @@ import bz2
 import gzip
 import io
 import json
+import lzma
 import pickle
 import re
 from typing import Union, Optional
-import lzma
+import warnings
 import pandas as pd
 from azure.identity import DefaultAzureCredential
 from azure.core.exceptions import ResourceNotFoundError
-from azfs.clients import AzfsClient
+from azfs.clients import AzfsClient, TextReader
 from azfs.error import AzfsInputError
 from azfs.utils import (
     BlobPathDecoder,
@@ -157,8 +158,6 @@ class AzFileClient:
 
         if credential is None and connection_string is None:
             credential = DefaultAzureCredential()
-        self.credential = credential
-        self.connection_string = connection_string
         self._client = AzfsClient(credential=credential, connection_string=connection_string)
 
     def __enter__(self):
@@ -248,7 +247,6 @@ class AzFileClient:
 
         """
         _, account_kind, _, file_path = BlobPathDecoder(path).get_with_url()
-        # file_list = AzfsClient.get(account_kind, credential=self.credential).ls(path=path, file_path=file_path)
         file_list = self._client.get_client(account_kind=account_kind).ls(path=path, file_path=file_path)
         if account_kind in ["dfs", "blob"]:
             file_name_list = ls_filter(file_path_list=file_list, file_path=file_path)
@@ -496,12 +494,14 @@ class AzFileClient:
     def du(self, path):
         pass
 
-    def _get(self, path: str, **kwargs) -> Union[bytes, str, io.BytesIO]:
+    def _get(self, path: str, offset: int = None, length: int = None, **kwargs) -> Union[bytes, str, io.BytesIO]:
         """
         get data from Azure Blob Storage.
 
         Args:
             path: Azure Blob path URL format, ex: ``https://testazfs.blob.core.windows.net/test_container/test1.csv``
+            offset:
+            length:
             **kwargs:
 
         Returns:
@@ -518,7 +518,89 @@ class AzFileClient:
 
         """
         _, account_kind, _, _ = BlobPathDecoder(path).get_with_url()
-        return self._client.get_client(account_kind=account_kind).get(path=path, **kwargs)
+
+        file_bytes = self._client.get_client(
+            account_kind=account_kind).get(path=path, offset=offset, length=length, **kwargs)
+        # gzip圧縮ファイルは一旦ここで展開
+        if path.endswith(".gz"):
+            file_bytes = gzip.decompress(file_bytes)
+
+        if type(file_bytes) is bytes:
+            file_to_read = io.BytesIO(file_bytes)
+        else:
+            file_to_read = file_bytes
+
+        return file_to_read
+
+    def read_line_iter(self, path: str) -> iter:
+        """
+        To read text file in each line with iterator.
+
+        Args:
+            path: Azure Blob path URL format, ex: ``https://testazfs.blob.core.windows.net/test_container/test1.csv``
+
+        Returns:
+            get data of the path as iterator
+
+        Examples:
+            >>> import azfs
+            >>> azc = azfs.AzFileClient()
+            >>> path = "https://testazfs.blob.core.windows.net/test_container/test1.csv"
+            >>> for l in azc.read_line_iter(path=path)
+            ...     print(l.decode("utf-8"))
+
+        """
+        _, account_kind, _, _ = BlobPathDecoder(path).get_with_url()
+        return TextReader(client=self._client.get_client(account_kind=account_kind), path=path)
+
+    def read_csv_chunk(self, path: str, chunk_size: int) -> pd.DataFrame:
+        """
+        !WARNING! the method may differ from current version in the future update.
+        Currently, only support for csv.
+
+        Args:
+            path: Azure Blob path URL format, ex: ``https://testazfs.blob.core.windows.net/test_container/test1.csv``
+            chunk_size: pandas-DataFrame index length to read.
+
+        Returns:
+            first time: len(df.index) is `chunk_size - 1`
+            second time or later: len(df.index) is `chunk_size`
+
+        Examples:
+            >>> import azfs
+            >>> azc = azfs.AzFileClient()
+            >>> path = "https://testazfs.blob.core.windows.net/test_container/test1.csv"
+            >>> chunk_size = 100
+            >>> for df in azc.read_csv_chunk(path=path, chunk_size=chunk_size):
+            ...   print(df)
+        """
+        warning_message = """
+            The method is under developing. 
+            The name or the arguments may differ from current version in the future update.
+        """
+        warnings.warn(warning_message, FutureWarning)
+        initial_line = ""
+        byte_list = []
+
+        for idx, l in enumerate(self.read_line_iter(path=path)):
+            div_idx = idx % chunk_size
+            if idx == 0:
+                initial_line = l
+                byte_list.append(initial_line)
+            else:
+                byte_list.append(l)
+            if div_idx + 1 == chunk_size:
+                file_to_read = (b"\n".join(byte_list))
+                file_to_io_read = io.BytesIO(file_to_read)
+                df = pd.read_csv(file_to_io_read)
+                yield df
+
+                byte_list = [initial_line]
+        # make remainder DataFrame after the for-loop
+        file_to_read = (b"\n".join(byte_list))
+        file_to_io_read = io.BytesIO(file_to_read)
+        df = pd.read_csv(file_to_io_read)
+        yield df
 
     @_az_context_manager.register(_as="read_csv_az", _to=pd)
     def read_csv(self, path: str, **kwargs) -> pd.DataFrame:
